/*
 *  Engine/DB.cs
 *  Copyright (C) 2005-2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


//#define devel


using MySql.Data.MySqlClient;
using System.Data;
using System;
using Mono.Unix;



namespace esofokles.Engine {
	public enum DBType {
		MySQL    = 0
	}


	public class DB {
		static IDbConnection Connection;
		static string        ConnectionStr;
		static bool          Connected;

		
		static public void Create (DBType type, string server, string user, string password, string port, string database)
		{
			switch (type) {
				case DBType.MySQL   : Connection = ConnectMySQL (server, user, password, port, database, "utf8"); break;
			}
		}


		static public bool Connect ()
		{
			try {
				Connection.Open ();
			} catch {
				Connected = false;
				return false;
			}
			
			
			Connected = true;
			return true;
		}


		static public ConnectionState State ()
		{
			if (Connection != null)
				return Connection.State;

			return ConnectionState.Closed;
		}


		static public bool Disconnect ()
		{
			if (State () == ConnectionState.Open) {
				Connection.Close ();
				Connected = false;
				return true;
			}
			
			return false;
		}


		static public IDataReader Query (string Query)
		{
			begin:
			
			IDbCommand cmd = Connection.CreateCommand ();
			cmd.CommandText = Query;

		#if devel
			Console.WriteLine ("@{0} >> {1}", DateTime.Now.ToString ("hh-mm-ss"), Query);
		#endif
			
			bool reconnect = false;
			try {
				IDataReader rdn = cmd.ExecuteReader ();
				cmd.Dispose ();
				return rdn;
			} catch (Exception e) {
				Console.WriteLine ("\n\n[QUERY] {0}", Query);
				Console.WriteLine ("[ERROR]  {0}", e.Message);
				
				int con = CheckConnection ();
				if (con == -1) {
					esofokles.Engine.Settings.Close ();
					Gtk.Application.Quit ();
				} else if (con == 1)
					reconnect = true;
				
				if (reconnect == false) {
					cmd.Dispose ();
					return null;
				}
			}
			
			if (reconnect == true) {
				try {
					Connection = new MySqlConnection (ConnectionStr);
					Connection.Open ();
				} catch { }
				
				goto begin;
			}
			
			return null;
		}


		static public bool FastQuery (string Query)
		{
			begin:
			
			IDbCommand cmd  = Connection.CreateCommand ();
			cmd.CommandText = Query;
			
			#if devel
				Console.WriteLine ("@{0} >> {1}", DateTime.Now.ToString ("hh-mm-ss"), Query);
			#endif
			
			bool reconnect = false;
			try {
				cmd.ExecuteNonQuery ();
				cmd.Dispose ();
			} catch (Exception e) {
				if (e.Message.StartsWith ("#23000Duplicate entry") == false) {
					Console.WriteLine ("\n\n[QUERY] {0}", Query);
					Console.WriteLine ("[ERROR]  {0}", e.Message);
				}	

				int con = CheckConnection ();
				if (con == -1) {
					esofokles.Engine.Settings.Close ();
					Gtk.Application.Quit ();
				} else if (con == 1)
					reconnect = true;
				
				if (reconnect == false) {
					cmd.Dispose ();
					return false;
				}
			}
			
			
			if (reconnect == true) {
				try {
					Connection = new MySqlConnection (ConnectionStr);
					Connection.Open ();
				} catch { }
				
				goto begin;
			}
			

			return true;
		}
		
		
		static private int CheckConnection ()
		{
			if (Connected == false)
				return 0;
			
			if ((Connection.State == ConnectionState.Closed) ||
								(Connection.State == ConnectionState.Broken)) {
				Gtk.MessageDialog msg = new Gtk.MessageDialog (null, Gtk.DialogFlags.DestroyWithParent, Gtk.MessageType.Error,
						Gtk.ButtonsType.YesNo, Catalog.GetString ("<b>Disconnected.</b> " +
						"There is a problem with the database connection.\n" +
						"Data or part of them which you has just entered may be deleted." +
						"Do you want to try to connect again ?"));
				if (msg.Run () == (int) Gtk.ResponseType.Yes) {
					msg.Destroy ();
					return 1;
				}
				
				msg.Destroy ();
				return -1;
			}
			
			return 0;
		}


		static private IDbConnection ConnectMySQL (string server, string user, string password, string port, string database, string charset)
		{
			string cnct = String.Format ("Database={0};UserID={1};Password={2};charset={3};", database, user, password, charset);
			
			if (server != "")
				cnct += "Server=" + server + ";";
			else
				cnct += "Server=localhost;";

			if (port != "")
				cnct += "Port=" + port + ";";

			cnct += "Pooling=false";
			
			ConnectionStr = cnct;
			return new MySqlConnection (cnct);
		}
		
		
		public static string Escape (string text)
		{
			return text.Replace ("\'", "\\\'");
		}
	}
}
