/*
 *  Engine/Tools.cs
 *  Copyright (C) 2005-2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.IO;
using Gdk;
using Gtk;


namespace esofokles.Engine {
	public class Tools {
		public static int CurrentSchoolYear ()
		{
			if (DateTime.Today.Month >= 7)
				return DateTime.Today.Year;
			else
				return DateTime.Today.Year - 1;
		}
		
		
		public static int GetCurrentSemester ()
		{
			if (((DateTime.Today.Month >= 7) && (DateTime.Today.Month <= 12)) ||
				(DateTime.Today.Month == 1))
				return 1;
			else
				return 2;
		}


		public static string SchoolYear (int year)
		{
			return String.Format ("{0} / {1}", year, year + 1);
		}


		public static int GetSchoolYear (string year)
		{
			string [] years = year.Split ('/');
			return System.Convert.ToInt16 (years [0]);
		}
		
		
		public static void SetSensitive (Gtk.Widget widget, bool sensitive)
		{
			widget.Sensitive = sensitive;
			
			while (Application.EventsPending ())
	        	Application.RunIteration ();
		}
		
		
		public static bool CheckPesel (string pesel)
		{
			int  s = 0, dec;
			bool completed = true;
			
			if (pesel.Length == 11)
				for (int i = 0; i < pesel.Length; i++) 
					try {
						dec = Convert.ToInt16 (pesel [i].ToString ());
						
						switch (i + 1) {
							case  1: s += (dec * 1); break;
							case  2: s += (dec * 3); break;
							case  3: s += (dec * 7); break;
							case  4: s += (dec * 9); break;
							case  5: s += (dec * 1); break;
							case  6: s += (dec * 3); break;
							case  7: s += (dec * 7); break;
							case  8: s += (dec * 9); break;
							case  9: s += (dec * 1); break;
							case 10: s += (dec * 3); break;
							case 11: s += (dec * 1); break;
						}
					} catch {
						completed = false;
						break;
					}
			else
				completed = false;
			
			
			if (completed == true)
				return (s.ToString () [s.ToString ().Length - 1] == '0');
			else
				return false;
		}
		
		
		/// <summary>Gets date from PESEL number. PESEL must be valid !</summary>
		public static string GetDateFromPesel (string pesel)
		{
			Console.WriteLine ("Getting");
			try {
				int year  = Convert.ToInt32 (pesel.Substring (0, 2));
				int month = Convert.ToInt32 (pesel.Substring (2, 2));
				int day   = Convert.ToInt32 (pesel.Substring (4, 2));
				
				if ((month >= 81) && (month <= 92)) {
					year  += 1800;
					month -=  80;
				} else if ((month >= 1) && (month <= 12))
					year  += 1900;
				else if ((month >= 21) && (month <= 32)) {
					year  += 2000;
					month -=  20;
				} else if ((month >= 41) && (month <= 52)) {
					year  += 2100;
					month -=  40;
				} else if ((month >= 61) && (month <= 72)) {
					year  += 2200;
					month -=  60;
				}
				
				return String.Format ("{0}-{1}-{2}", day, month, year);
			} catch {
				return "";
			}
		}
		
		
		public static void RunWebBrowser (string Url)
		{
			 System.Diagnostics.Process.Start (GetInternetBrowser (), Url);
		}
		
		
		private static string GetInternetBrowser ()
		{
			if (FilesPaths.IsWinSystem () == true)
				return "iexplore.exe";
			else {
				foreach (string path in System.Environment.GetEnvironmentVariable ("PATH").Split (':'))
					if (File.Exists (path + "/firefox"))        return path + "/firefox";
					else if (File.Exists (path + "/epiphany"))  return path + "/epiphany";
					else if (File.Exists (path + "/mozilla"))   return path + "/mozilla";
					else if (File.Exists (path + "/konqueror")) return path + "/konqueror";
					else if (File.Exists (path + "/opera"))     return path + "/opera";
				
				return "";
			}
		}
	}
}
