/*
 *  GUI/AddingMarksDialog.cs
 *  Copyright (C) 2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Data;
using System.Collections;
using esofokles.Engine;
using esofokles.Widgets;
using Gtk;
using Mono.Unix;
using Glade;



namespace esofokles.GUI {

	/// <summary>About dialog</summary>
	public class AddingMarksDialog {
		public Gtk.Dialog dialog;
		
		[Glade.Widget] Gtk.Entry       Comment, Mark;
		[Glade.Widget] Gtk.ComboBox    Subjects, Students;
		[Glade.Widget] Gtk.Button      AddBtn, RemoveBtn, CancelBtn, CloseBtn;
		[Glade.Widget] Gtk.TreeView    Marks;
		[Glade.Widget] Gtk.VBox        DateVBox;
		[Glade.Widget] Gtk.Notebook    Tabs;
		[Glade.Widget] Gtk.RadioButton FirstSemester, SecondSemester;
		
		DateEdit MarkDate;
		Button24 SaveMarksBtn, AddNewMarksBtn;
		
		
		public struct Student {
			public int    ID;
			public string Name;
			
			public Student (int id, string name)
			{
				ID = id;
				Name = name;
			}
		}
		
		ArrayList StudentsList, SubjectsList; 
		
		/// <summary>Constructor. Creates about dialog.</summary>
		public AddingMarksDialog (Gtk.Window parent, int ClassID)
		{
			Glade.XML gui = Glade.XML.FromAssembly ("gui.glade", "AddingMarksDialog", null);
			gui.Autoconnect (this);
		
			dialog = (Gtk.Dialog) gui.GetWidget ("AddingMarksDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;
			
			
			StudentsList = new ArrayList ();
			SubjectsList = new ArrayList ();
			
			
			// SaveMarksBtn
			SaveMarksBtn = new Button24 (Catalog.GetString ("Save marks"));
			SaveMarksBtn.Clicked += new EventHandler (on_SaveBtn_clicked);
			dialog.ActionArea.Add (SaveMarksBtn);
			SaveMarksBtn.Sensitive = false;
			SaveMarksBtn.ShowAll ();
			
			
			// AddNewMarksBtn
			AddNewMarksBtn = new Button24 (Catalog.GetString ("Add new marks"));
			AddNewMarksBtn.Clicked += new EventHandler (on_AddNewMarksBtn_clicked);
			dialog.ActionArea.Add (AddNewMarksBtn);
			AddNewMarksBtn.ShowAll ();
			AddNewMarksBtn.Hide ();
			
			
			// Icons and colors
			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.ADD_MARK_BUTTON, "Icon", gui);
			Icons.AssignImage (Icons.CANCEL_BUTTON, "CancelBtnImg", gui);
			Icons.AssignImage (Icons.CLOSE_BUTTON, "CloseBtnImg", gui);
			Icons.AssignImage (Icons.ADD_MARK_BUTTON, AddNewMarksBtn.Img);
			Icons.AssignImage (Icons.SAVE_MARK_BUTTON, SaveMarksBtn.Img);
			
			Colors.Modify (Colors.HEAD, "Bg", gui);
			Colors.Modify (Colors.BG, "Msg1Bg", gui);
			Colors.Modify (Colors.BG, "Msg2Bg", gui);
			
			
			// MarkDate
			MarkDate = new DateEdit ();
			DateVBox.PackStart (MarkDate, true, true, 0);
			
			
			// Loading students list
			IDataReader reader  = DB.Query (String.Format ("SELECT * FROM " + 
						"students WHERE class = '{0}' ORDER by surname;", ClassID));
			if (reader != null) {
				while (reader.Read ()) {
					Students.AppendText ((string) reader ["name"] + " " +
											(string) reader ["surname"]);
					
					StudentsList.Add (new Student ((int) reader ["id"], (string)
							reader ["name"] + " " + (string) reader ["surname"]));
				}
				
				reader.Close ();
			}
			
			if (Students.Model.IterNChildren () > 0)
				Students.Active = 0;
			else {
				Tabs.Page = 1;
				SaveMarksBtn.Hide ();
				CancelBtn.Hide ();
				CloseBtn.Show ();
				
				return;
			}
			

			// Loading subjects list
			reader = DB.Query (String.Format ("SELECT * FROM " + 
						"subjects WHERE class = '{0}' ORDER by subject;", ClassID));
			if (reader != null) {
				while (reader.Read ()) {
					Subjects.AppendText ((string) reader ["subject"]);
					SubjectsList.Add ((int) reader ["id"]);
				}
				
				reader.Close ();
			}
			
			if (Subjects.Model.IterNChildren () > 0)
				Subjects.Active = 0;
			
			
			// Creating Marks treeview
			Gtk.ListStore store = new Gtk.ListStore (typeof (int), typeof (string), typeof (string));
			Marks.Model = store;

			Marks.AppendColumn (Catalog.GetString ("Name"), new CellRendererText  (), "text", 1);
			Marks.AppendColumn (Catalog.GetString ("Mark"), new CellRendererText (), "text", 2);
			
			
			// Semester
			if (Tools.GetCurrentSemester () == 1)
				FirstSemester.Active = true;
			else
				SecondSemester.Active = true;
			
			
			Autocompletion.Assign (Comment, Field.MarksComment);
			dialog.Focus = Mark;
		}
		
		
		protected void on_SaveBtn_clicked (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			Autocompletion.AddText (Comment, Field.MarksComment);
			
			SaveMarksBtn.Hide ();
			CancelBtn.Hide ();
			CloseBtn.Show ();
			AddNewMarksBtn.Show ();
			
			// inserting marks
			Gtk.ListStore store = Marks.Model as ListStore;
			Gtk.TreeIter  iter  = TreeIter.Zero;
			string        query = "";
			
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				int sem = 0;
				if (FirstSemester.Active == false)
					sem = 1;
				
				query += String.Format ("INSERT INTO marks (student, semester, subject, mark, date, description) VALUES('{0}', " +
						"'{1}', '{2}', '{3}', '{4}', '{5}');",
						(int) store.GetValue (iter, 0), sem,
						SubjectsList [Subjects.Active],
						DB.Escape ((string) store.GetValue (iter, 2)),
						MarkDate.GetDateString (), DB.Escape (Comment.Text));
			}
			
			DB.FastQuery (query);
			
			
			Tabs.Page = 2;
			Tools.SetSensitive (dialog, true);
			
			dialog.Focus = AddNewMarksBtn;
		}
		
		
		protected void on_AddNewMarksBtn_clicked (object o, EventArgs args)
		{
			CloseBtn.Hide ();
			AddNewMarksBtn.Hide ();
			SaveMarksBtn.Show ();
			CancelBtn.Show ();
			
			// Cleaning
			Gtk.TreeIter  itr;
			
			while (Marks.Model.IterNChildren () > 0)
				if (Marks.Model.GetIterFirst (out itr)) {
					Students.AppendText ((string) (Marks.Model as ListStore).GetValue (itr, 1));
					(Marks.Model as ListStore).Remove (ref itr);
					
					if (Students.Model.IterNChildren () > 0)
						Students.Active = 0;
					
					RemoveBtn.Sensitive = false;
					SaveMarksBtn.Sensitive = Marks.Model.IterNChildren () > 0;
				}
			
			
			Tabs.Page = 0;
			dialog.Focus = Mark;
		}

		
		protected void on_Marks_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			RemoveBtn.Sensitive = Marks.Selection.GetSelected (out mod, out itr);
		}
		
		
		protected void on_Students_changed (object o, EventArgs args)
		{
			Mark.Sensitive = Students.Model.IterNChildren () > 0;
			AddBtn.Sensitive = Students.Model.IterNChildren () > 0;
			Students.Sensitive = Students.Model.IterNChildren () > 0;
			Mark.Sensitive = Students.Model.IterNChildren () > 0;
		}
		
		
		protected void on_Mark_activate (object o, EventArgs args)
		{
			if (AddBtn.Sensitive)
				on_AddBtn_clicked (null, null);
		}
		
		
		protected void on_Mark_changed (object o, EventArgs args)
		{
			AddBtn.Sensitive = (Mark.Text.Length > 0) && (Students.Active >= 0);
		}
		
		
		protected void on_AddBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeIter iter = Gtk.TreeIter.Zero;
			Students.GetActiveIter (out iter);
			
			foreach (Student snt in StudentsList)
				if (snt.Name == (string) Students.Model.GetValue (iter, 0)) {
					(Marks.Model as ListStore).AppendValues (new object []
										{ snt.ID, snt.Name, Mark.Text});
					
					(Students.Model as ListStore).Remove (ref iter);
					AddBtn.Sensitive = Students.Model.IterNChildren () > 0;
					Students.Sensitive = Students.Model.IterNChildren () > 0;
					Mark.Sensitive = Students.Model.IterNChildren () > 0;
					
					Mark.Text = "";
					
					if (Students.Model.IterNChildren () > 0)
						Students.Active = 0;
					
					if (Students.Model.IterNChildren () > 0)
						dialog.Focus = Mark;
					else
						dialog.Focus = SaveMarksBtn;
					
					SaveMarksBtn.Sensitive = Marks.Model.IterNChildren () > 0;
					
					break;
				}
		}
		
		
		protected void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (Marks.Selection.GetSelected (out mod, out itr)) {
				Students.AppendText ((string) (mod as ListStore).GetValue (itr, 1));
				(mod as ListStore).Remove (ref itr);
				
				if (Students.Model.IterNChildren () > 0)
					Students.Active = 0;
				
				RemoveBtn.Sensitive = false;
				SaveMarksBtn.Sensitive = Marks.Model.IterNChildren () > 0;
			}
		}
	}
}