/*
 *  GUI/ImportDialog.cs
 *  Copyright (C) 2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */
 

using System;
using System.Reflection;
using System.Data;
using System.Collections;
using Mono.Unix;
using Gdk;
using Gtk;
using Glade;
using esofokles.Engine;
using esofokles.Widgets;



namespace esofokles.GUI {

	/*
	 * ImportClassesDialog class
	 */
	
	public class ImportClassesDialog {
		public Gtk.Dialog dialog;

		[Glade.Widget] private Gtk.Button      LeftBtn, RightBtn, CancelBtn;
		[Glade.Widget] private Gtk.Notebook    Tabs;
		[Glade.Widget] public  Gtk.TreeView    PreviousSchoolYear, CurrentSchoolYear;
		[Glade.Widget] private Gtk.VBox        ImportClassesDialogVBox;

		public  int       SchoolYear;
		private Button24  ImportBtn;

		
		public ImportClassesDialog (Gtk.Window parent, int SchoolYear)
		{
			Glade.XML gui = Glade.XML.FromAssembly ("gui.glade", "ImportClassesDialog", null);
			gui.Autoconnect (this);
			
			dialog = (Gtk.Dialog) gui.GetWidget ("ImportClassesDialog");
			dialog.TransientFor = parent;
			dialog.Icon = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "esofokles.png");
			dialog.DestroyWithParent = true;
			ImportClassesDialogVBox.BorderWidth = 0;
			
			
			// Creating ImportButton
			ImportBtn          = new Button24 (Catalog.GetString ("_Import classes"));
			ImportBtn.Clicked += new EventHandler (on_ImportBtn_clicked);
			ImportBtn.ShowAll ();
			dialog.ActionArea.Add (ImportBtn);
			

			// Icons and colors
			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.IMPORT_CLASSES_DIALOG, "Icon", gui);
			Icons.AssignImage (Icons.IMPORT_CLASSES_BUTTON, ImportBtn.Img);
			Icons.AssignImage (Icons.CANCEL_BUTTON, "CancelBtnImg", gui);
			Icons.AssignImage (Icons.LEFT_BUTTON, "LeftArrowImg", gui);
			Icons.AssignImage (Icons.RIGHT_BUTTON, "RightArrowImg", gui);
			
			Colors.Modify (Colors.HEAD, "Bg", gui);
			
			
			// Message
			new MessageBox (MessageBox.MsgType.info, false, "<span size='small'>" +
				Catalog.GetString ("To import classes from previous school year, " +
				"select them on <u>Previous school year</u> list. Then click on " +
				"right arrow or just double click on selected class. Lastly click " +
				"on <u>Import classes</u> button.") + "</span>").AddToBox ("ImportBox", gui);
			
	
			// Creating Previous school year list
			Gtk.ListStore store = new Gtk.ListStore (typeof (string), typeof (int));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			PreviousSchoolYear.Model = store;
			PreviousSchoolYear.AppendColumn ("", new CellRendererText  (), "text", 0);

			// Creating current school year list
			Gtk.ListStore store2 = new Gtk.ListStore (typeof (string), typeof (int));
			store2.SetSortColumnId (0, Gtk.SortType.Ascending);
			CurrentSchoolYear.Model = store2;
			CurrentSchoolYear.AppendColumn ("", new CellRendererText  (), "text", 0);

			
			// sensitive
			Tools.SetSensitive (dialog, false);
			
			// creating all classes list
			ArrayList   arr     = new ArrayList ();
			IDataReader reader  = DB.Query (String.Format ("SELECT * FROM classes " + 
										"WHERE year = '{0}';", SchoolYear + 1));
			if (reader != null) {
				while (reader.Read ())
					arr.Add (String.Format ("{0} {1}", (int) reader ["number"],
												(string) reader ["token"]));
				
				reader.Close ();
			}

			
			bool w = false;
			IDataReader reader2  = DB.Query (String.Format ("SELECT * FROM " + 
									"classes WHERE year = '{0}';", SchoolYear));
			if (reader2 != null) {
				while (reader2.Read ()) {
					foreach (string item in	arr) {
						if (String.Format ("{0} {1}", (int) reader2 ["number"] + 1,
										(string) reader2 ["token"]) == item)
							goto ok;
					}
					
					store.AppendValues (new object [] {(int) reader2 ["number"] +
												" " + (string) reader2 ["token"],
									(int) reader2 ["id"]});
					w = true;
					
					ok: ;
				}
				
				reader2.Close ();
			}


			this.SchoolYear = SchoolYear;
			
			if (w == false) {
				Tabs.Page = 3;
				ImportBtn.Hide ();
			}
			
			Tools.SetSensitive (dialog, true);
		}
		
		
		protected void on_ImportBtn_clicked (object o, EventArgs args)
		{
			if (Tabs.Page == 2) {
				dialog.Respond (ResponseType.Ok);
				return;
			}
			

			Tabs.Page = 1;
			Tools.SetSensitive (dialog, false);
			
			Gtk.TreeIter  iter;
			Gtk.ListStore store = (Gtk.ListStore) CurrentSchoolYear.Model;

			
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				DB.FastQuery (String.Format ("INSERT INTO classes (year, description" +
							", number, token) (select year + 1, description, " +
							"number + 1, token from classes where id = '{0}');",
							(int) store.GetValue (iter, 1)));
				
				int CID = SearchClass ((string) store.GetValue (iter, 0), SchoolYear);
				DB.FastQuery (String.Format ("INSERT INTO subjects (class, subject, " +
							"teacher, average) (select '{0}', subject, teacher, " +
							"average from subjects where class = '{1}');", CID,
							(int) store.GetValue (iter, 1)));
				DB.FastQuery (String.Format ("INSERT INTO students (class, name, " +
							"surname, pesel, sid, birthdate, birthplace, address, " +
							"parentsaddress, telephone, father, mother, info) " +
							"(select '{0}', name, surname, pesel, sid, birthdate, " +
							"birthplace, address, parentsaddress, telephone, father, " +
							"mother, info from students where class = '{1}');", CID,
							(int) store.GetValue (iter, 1)));
			}
			
			ImportBtn.Lbl.Text = Catalog.GetString ("Finish");
			Icons.AssignImage (Icons.RIGHT_BUTTON, ImportBtn.Img);
			
			CancelBtn.Hide ();
			
			Tabs.Page = 2;
			Tools.SetSensitive (dialog, true);
		}
		
		
		/// <summary>Searchs class by name and school year.</summary>
		private int SearchClass (string ClassFullName, int year)
		{
			string token  = "";
			int    number = 0;
			
			for (int i = 0; i < ClassFullName.Length; i++)
				if (ClassFullName [i] == ' ') {
					number = Convert.ToInt32 (ClassFullName.Substring (0, i));
					token  = ClassFullName.Substring (i + 1, ClassFullName.Length - i - 1);
					break;
				}
		
			
			IDataReader reader = DB.Query (String.Format ("SELECT * FROM classes " +
									"WHERE number = '{0}' AND token = '{1}' AND " +
									"year = '{2}';", number + 1, DB.Escape (token), year + 1));
			
			if (reader != null) {
				if (reader.Read ()) {
					int id = (int) reader ["id"];
					reader.Close ();
					return id;
				}
				
				reader.Close ();
			}
			
			return -1;
		}



		protected void on_PreviousSchoolYear_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			RightBtn.Sensitive = PreviousSchoolYear.Selection.GetSelected (out mod, out itr);
		}


		protected void on_CurrentSchoolYear_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			LeftBtn.Sensitive = CurrentSchoolYear.Selection.GetSelected (out mod, out itr);
		}


		protected void on_PreviousSchoolYear_row_activated (object o, RowActivatedArgs args)
		{
			on_RightBtn_clicked (null, null);
		}


		protected void on_CurrentSchoolYear_row_activated (object o, RowActivatedArgs args)
		{
			on_LeftBtn_clicked (null, null);
		}


		protected void on_RightBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (PreviousSchoolYear.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore pStore = (Gtk.ListStore) PreviousSchoolYear.Model;
				Gtk.ListStore cStore = (Gtk.ListStore) CurrentSchoolYear.Model;

				cStore.AppendValues (new object [] {(string) pStore.GetValue (itr, 0),
												(int) pStore.GetValue (itr, 1)});
				pStore.Remove (ref itr);

				RightBtn.Sensitive = false;
			}
		}


		protected void on_LeftBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (CurrentSchoolYear.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore pStore = (Gtk.ListStore) PreviousSchoolYear.Model;
				Gtk.ListStore cStore = (Gtk.ListStore) CurrentSchoolYear.Model;

				pStore.AppendValues (new object [] {(string) cStore.GetValue (itr, 0),
												(int) cStore.GetValue (itr, 1)});
				cStore.Remove (ref itr);

				LeftBtn.Sensitive = false;
			}
		}
	}
}
