/*
 *  GUI/ReportsDialog.cs
 *  Copyright (C) 2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Collections;
using System.IO;
using System.Data;
using System.Data.SqlTypes;
using esofokles.Engine;
using Gtk;
using Gdk;
using esofokles.Widgets;
using Mono.Unix;


namespace esofokles.GUI {
	
	/*
	 * ReportsDialog class
	 */
	
	public class ReportsDialog {
		public Gtk.Dialog dialog;

		[Glade.Widget] private Gtk.Button      CloseBtn;
		[Glade.Widget] private Gtk.RadioButton ReportOneStudent;
		[Glade.Widget] private Gtk.ComboBox    ReportStudent, Report;
		[Glade.Widget] private Gtk.CheckButton PrintAttend, PrintRemarks;
		[Glade.Widget] private Gtk.Notebook    Pages;

		private ArrayList StudentsList;
		private Button24  MakeReportBtn, NewReportBtn, ShowReport;
		private string    ReportUrl, ClassName, EducatorName;
		
		
		public ReportsDialog (Gtk.Window parent, int ClassID)
		{
			Glade.XML gui = Glade.XML.FromAssembly ("gui.glade", "ReportsDialog", null);
			gui.Autoconnect (this);
			
			dialog = (Gtk.Dialog) gui.GetWidget ("ReportsDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;
			dialog.DestroyWithParent = true;
			
			
			if (Tools.GetCurrentSemester () == 1)
				Report.Active = 0;
			else
				Report.Active = 1;
			
			
			// MakeReportBtn
			MakeReportBtn = new Button24 (Catalog.GetString ("Make report"));
			MakeReportBtn.Clicked += new EventHandler (on_MakeReportBtn_clicked);
			MakeReportBtn.Sensitive = false;
			MakeReportBtn.ShowAll ();
			dialog.ActionArea.PackStart (MakeReportBtn, false, false, 6);
			
			// Show report button
			ShowReport = new Button24 (Catalog.GetString ("Show report"));
			ShowReport.Clicked += new EventHandler (on_ShowReport_clicked);
			dialog.ActionArea.PackStart (ShowReport, false, false, 6);
			
			// New report button
			NewReportBtn = new Button24 (Catalog.GetString ("New report"));
			NewReportBtn.Clicked += new EventHandler (on_NewReportBtn_clicked);
			dialog.ActionArea.PackStart (NewReportBtn, false, false, 6);
			
			
			// Icons and colors
			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.REPORTS_DIALOG, "Icon", gui);
			Icons.AssignImage (Icons.CLOSE_BUTTON, "CloseBtnImg", gui);
			Icons.AssignImage (Icons.REPORTS_BUTTON, MakeReportBtn.Img);
			Icons.AssignImage (Icons.REPORTS_BUTTON, NewReportBtn.Img);
			
			Colors.Modify (Colors.HEAD, "Bg", gui);

			Tools.SetSensitive (dialog, false);
			
			
			// Students list
			StudentsList = new ArrayList ();
			IDataReader reader2 = DB.Query (String.Format ("SELECT * FROM students " +
							"WHERE class = '{0}' ORDER BY surname;", ClassID));
			bool ok = false;
			if (reader2 != null) {
				while (reader2.Read ()) {
					ReportStudent.AppendText ((string) reader2 ["name"] + " " +
												(string) reader2 ["surname"]);
					StudentsList.Add ((int) reader2 ["id"]);
					ok = true;
				}

				Gtk.TreeIter iter;
				if (ReportStudent.Model.GetIterFirst (out iter))
					ReportStudent.Active = 0;

				reader2.Close ();
			}

			
			if (ok == false) {
				Pages.Page = 1;
				Tools.SetSensitive (dialog, true);
				MakeReportBtn.Hide ();
				ShowReport.Hide ();
				dialog.Focus = CloseBtn;
			} else {
				Tools.SetSensitive (dialog, true);
				dialog.Focus = PrintAttend;
			}
			
			// Class Info
			IDataReader reader3 = DB.Query (String.Format ("SELECT number,token,description FROM classes " +
							"WHERE id = '{0}';", ClassID));
			if (reader3 != null) {
				if (reader3.Read ()) {
					ClassName    = (int) reader3 ["number"] + " " + (string) reader3 ["token"];
					EducatorName = (string) reader3 ["description"];
				}
				
				reader3.Close ();
			}
		}
		
		
		protected void on_ReportStudent_changed (object o, EventArgs args)
		{
			if (ReportOneStudent.Active == true)
				MakeReportBtn.Sensitive = ReportStudent.Active >= 0;
			else
				MakeReportBtn.Sensitive = true;
		}


		protected void on_ReportOneStudent_toggled (object o, EventArgs args)
		{
			if (ReportOneStudent.Active == true)
				MakeReportBtn.Sensitive = ReportStudent.Active >= 0;
			else
				MakeReportBtn.Sensitive = true;
			
			ReportStudent.Sensitive = ReportOneStudent.Active;
		}
		
		
		protected void on_NewReportBtn_clicked (object o, EventArgs args)
		{
			NewReportBtn.Hide ();
			ShowReport.Hide ();
			MakeReportBtn.Show ();
			Pages.Page = 0;
		}
		
		
		protected void on_ShowReport_clicked (object o, EventArgs args)
		{
			Tools.RunWebBrowser (ReportUrl);
		}


		protected void on_MakeReportBtn_clicked (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			bool ShowMsg = false;
			
			if (ReportOneStudent.Active) {
				Gtk.FileChooserDialog dlg = new Gtk.FileChooserDialog (Catalog.GetString (
								"Report file"), dialog, FileChooserAction.Save);
				string Direct = Engine.Settings.Get ("ReportsFileDirectory");
				if (System.IO.Directory.Exists (Direct))
					dlg.SetCurrentFolder (Direct);
				
				dlg.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
				dlg.AddButton (Gtk.Stock.Save, ResponseType.Ok);

				Gtk.TreeIter itr;
				ReportStudent.GetActiveIter (out itr);
				dlg.CurrentName = ((string) ReportStudent.Model.GetValue (itr, 0) + ".html").Replace (' ', '_');
			
				if (dlg.Run () == (int) ResponseType.Ok) {
					dlg.Hide ();
					
					Pages.Page = 2;
					Tools.SetSensitive (dialog, true);
					Tools.SetSensitive (dialog, false);
					
					MakeReport ((int) StudentsList [ReportStudent.Active], dlg.Filename);
					ReportUrl = dlg.Filename;
					ShowMsg = true;
				}
				
				Engine.Settings.Set ("ReportsFileDirectory", System.IO.Path.GetDirectoryName (dlg.Filename));
				dlg.Destroy ();
			} else {
				Gtk.FileChooserDialog dlg = new Gtk.FileChooserDialog (Catalog.GetString (
						"Select reports directory"), dialog, FileChooserAction.SelectFolder);
				
				string Direct = Engine.Settings.Get ("ReportsDirectory");
				if (System.IO.Directory.Exists (Direct))
					dlg.SetCurrentFolder (Direct);
				
				dlg.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
				dlg.AddButton (Gtk.Stock.Open, ResponseType.Ok);
				
				if (dlg.Run () == (int) ResponseType.Ok) {
					dlg.Hide ();
					
					Pages.Page = 2;
					Tools.SetSensitive (dialog, true);
					Tools.SetSensitive (dialog, false);
					
					Gtk.TreeIter iter;
					int          i = 0;
					ArrayList    LinksList = new ArrayList ();
					for (bool tmp = ReportStudent.Model.GetIterFirst (out iter);
						tmp == true; tmp = ReportStudent.Model.IterNext (ref iter)) {
						MakeReport ((int) StudentsList [i], dlg.CurrentFolder +
									Engine.FilesPaths.Separator () +
									(ReportStudent.Model.GetValue (iter, 0) as string).Replace (' ', '_') + "." + (i + 1) + ".html");
						
						LinksList.Add ("<a href=\"" + (ReportStudent.Model.GetValue (iter, 0) as string).Replace (' ', '_') + "." + (i + 1) +
								".html\">" + (ReportStudent.Model.GetValue (iter, 0) as string) + "</html>");
						i++;
					}
					
					MakeIndex (LinksList, dlg.CurrentFolder + Engine.FilesPaths.Separator () + "index.html");
					ShowMsg = true;
				}
				
				Engine.Settings.Set ("ReportsDirectory", System.IO.Path.GetDirectoryName (dlg.Filename));
				dlg.Destroy ();
			}

			if (ShowMsg == true) {
				Pages.Page = 3;
				MakeReportBtn.Hide ();
				ShowReport.Show ();
				NewReportBtn.ShowAll ();
			}
			
			Tools.SetSensitive (dialog, true);
		}
		
		
		/// <summary>Makes index.html file with links to other students reports.</summary>
		private void MakeIndex (ArrayList LinksList, string FileName)
		{
			string src = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transit"
					+ "ional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">"
					+ "<html xmlns=\"http://www.w3.org/1999/xhtml\">"
					+ "<head>"
					+ "\t<title>" + Catalog.GetString ("Reports list") + "</title>"
					+ "\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />"
					+ "\t<meta name=\"generator\" content=\"e-Sofokles\" />"
					+ "</head>"
					+ "<body>"
					+ "<b>" + Catalog.GetString ("Students") + "</b>" + ":<br/>";

			foreach (string lnk in LinksList)
				src += lnk + "<br/>\n";

			src += "</body>\n</html>";

			StreamWriter file = new StreamWriter (FileName, false, System.Text.Encoding.UTF8);
			file.Write (src);
			file.Close ();
			
			ReportUrl = FileName;
		}


		/// <summary>Makes report</summary>
		private void MakeReport (int StudentID, string FileName)
		{
			string sName = "";
			IDataReader reader2 = DB.Query (String.Format ("SELECT * FROM students " +
											"WHERE id = '{0}';", StudentID));

			if (reader2 != null) {
				if (reader2.Read ())
					sName = (string) reader2 ["name"] + " " + (string) reader2 ["surname"];

				reader2.Close ();
			}
			
			
			int SemesterNumber = Report.Active + 1;

			string src = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transit"
					+ "ional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">"
					+ "<html xmlns=\"http://www.w3.org/1999/xhtml\">"
					+ "<head>"
					+ "\t<title>" + Catalog.GetString ("Report") + "</title>"
					+ "\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />"
					+ "\t<meta name=\"generator\" content=\"e-Sofokles\" />"
					+ "</head>"
					+ "<body>";

			src += "<div style='text-align: right; font-weight: bold; font-style: italic;'>" +
					String.Format (Catalog.GetString ("The report was generated on {0} at {1}."),
						System.DateTime.Now.ToString ("dd-MM-yyyy"), System.DateTime.Now.ToString ("HH:mm")) +
					"</div>";
			src += "<h1>" + sName + "</h1>";
			src += "<b>" + Catalog.GetString ("Class") + "</b>: " + ClassName + "<br/>";
			if (EducatorName != "")
				src += "<b>" + Catalog.GetString ("Educator") + "</b>: " + EducatorName + "<br/>";
			src += "<b>" + Catalog.GetString ("Semester") + "</b>: " + SemesterNumber + "<br/>";
			src += GetMarks (StudentID);
			src += GetAttendance (StudentID);
			src += GetRemarks (StudentID);

			src += "</body>\n</html>";

			StreamWriter file = new StreamWriter (FileName, false, System.Text.Encoding.UTF8);
			file.Write (src);
			file.Close ();
		}


		/// <summary>Subject info structure</summary>
		private struct Subject {
			public string Name;
			public string Marks;
			public string Description;
			public int    SemesterMark;


			public Subject (string name, string marks, int smark, string description)
			{
				Name = name;
				Marks = marks;
				SemesterMark = smark;
				Description = description;
			}
		}


		/// <summary>Gets marks</summary>
		private string GetMarks (int StudentID)
		{
			IDataReader reader;
			ArrayList subjects = new ArrayList ();
			string rtn = "";
			bool PrintMarks = false, PrintSemesterMarks = false, PrintDescriptiveMarks = false;
			
			reader = DB.Query (String.Format ("SELECT marks.mark,subjects.subject " +
						"FROM marks,subjects WHERE student = '{0}' AND semester = " +
						"'{1}' AND subjects.id = marks.subject ORDER BY 'subject';",
						StudentID, Report.Active));
			if (reader != null) {
				while (reader.Read ()) {
					PrintMarks = true;
					
					for (int i = 0; i < subjects.Count; i++) {
						Subject s = (Subject) subjects [i];
						if (s.Name == (string) reader ["subject"]) {
							s.Marks += ", " + (string) reader ["mark"];
							subjects [i] = s;
							goto n;
						}
					}
					
					Subject sbj = new Subject ((string) reader ["subject"],
												(string) reader ["mark"], -3, "");
					subjects.Add (sbj);
					n:
					;
				}
				
				reader.Close ();
			}
			
			reader = DB.Query (String.Format ("SELECT semestermarks.mark, semestermarks.description,subjects." + 
							"subject FROM semestermarks,subjects WHERE student " +
							"= '{0}' AND semester = '{1}' AND subjects.id = " +
							"semestermarks.subject ORDER BY 'subject'",
							StudentID, Report.Active));
			if (reader != null) {
				while (reader.Read ()) {
					if ((int) reader ["mark"] == -4)
						PrintDescriptiveMarks = true;
					else if ((int) reader ["mark"] != -1)
						PrintSemesterMarks = true;
					
					for (int i = 0; i < subjects.Count; i++) {
						Subject s = (Subject) subjects [i];
						if (s.Name == (string) reader ["subject"]) {
							s.SemesterMark = (int) reader ["mark"];
							subjects [i] = s;
							goto o;
						}
					}

					Subject sbj = new Subject ((string) reader ["subject"], "",
								(int) reader ["mark"], (string) reader ["description"]);
					subjects.Add (sbj);

					o:
					;
				}
				
				reader.Close ();
			}
			
			
			Subject BehMark = new Subject (Catalog.GetString ("Behaviour"), "", -3, "");
			reader = DB.Query (String.Format ("SELECT mark FROM semestermarks " +
					"WHERE student = '{0}' AND semester = '{1}' AND " +
					"subject = '0'", StudentID, Report.Active));
			string beh = "";
			if (reader != null) {
				if (reader.Read ()) {
					BehMark.SemesterMark = (int) reader ["mark"];
					PrintSemesterMarks   = true;
				}
				
				reader.Close ();
			}
			
			reader = DB.Query (String.Format ("SELECT mark FROM marks " +
					"WHERE student = '{0}' AND semester = '{1}' AND " +
					"subject = '0'", StudentID, Report.Active));
			int j  = 0;
			if (reader != null) {
				while (reader.Read ()) {
					if (j == 0)
						BehMark.Marks += (string) reader ["mark"];
					else
						BehMark.Marks += ", " + (string) reader ["mark"];
					
					j++;
				}
				
				PrintMarks = j > 0;
				
				reader.Close ();
			}
			
			subjects.Add (BehMark);


			foreach (Subject sub in subjects) {
				string sem = "";
				
				if (sub.Name != Catalog.GetString ("Behaviour"))
					switch (sub.SemesterMark) {
						case  6: sem = Catalog.GetString ("Excellent");     break;
						case  5: sem = Catalog.GetString ("Very good");     break;
						case  4: sem = Catalog.GetString ("Good");          break;
						case  3: sem = Catalog.GetString ("Sufficient");    break;
						case  2: sem = Catalog.GetString ("Poor");          break;
						case  1: sem = Catalog.GetString ("Insufficient");  break;
						case -1: sem = "";                                  break;
						case -2: sem = Catalog.GetString ("Discharged");    break;
						case -3: sem = Catalog.GetString ("Ungraded");      break;
					}
				else
					switch (sub.SemesterMark) {
						case  6: sem = Catalog.GetString ("Wzorowe");        break;
						case  5: sem = Catalog.GetString ("Bardzo dobre");   break;
						case  4: sem = Catalog.GetString ("Dobre");          break;
						case  3: sem = Catalog.GetString ("Poprawne");       break;
						case  2: sem = Catalog.GetString ("Nieodpowiednie"); break;
						case  1: sem = Catalog.GetString ("Naganne");        break;
						default: sem = ""; break;
					}
				
				
				string MarksStr = "", SemesterMarksStr = "", DescriptiveMarks = "";
				
				if (PrintMarks == true)
					MarksStr = "<td>" + sub.Marks + "</td>";
				
				if (PrintSemesterMarks == true)
					SemesterMarksStr = "<td>" + sem + "</td>";
				
				if (PrintDescriptiveMarks == true)
					DescriptiveMarks = "<td>" + sub.Description + "</td>";
				
				rtn += String.Format ("<tr><td><b>{0}</b></td>{1}{2}{3}</tr>",
									sub.Name, MarksStr, SemesterMarksStr, DescriptiveMarks);
			}
			
			
			if ((PrintMarks == true) || (PrintSemesterMarks == true) || (PrintDescriptiveMarks == true)) {
				string MarksStr = "", SemesterMarksStr = "", DescriptiveMarks = "", BehaviourMarkStr = "";
				
				if (PrintMarks == true)
					MarksStr         = "<td><b>" + Catalog.GetString ("Marks") + "</b></td>";
				
				if (PrintSemesterMarks == true)
					SemesterMarksStr = "<td><b>" + Catalog.GetString ("Semester mark") + "</b></td>";
				
				if (PrintDescriptiveMarks == true)
					DescriptiveMarks = "<td><b>" + Catalog.GetString ("Descriptive mark") + "</b></td>";
				
				return "<h2>" + Catalog.GetString ("Marks") + "</h2>" +
						"<table style=\"border: 1px none #333333;\" border=\"1\" cellpadding=\"6\" cellspacing=\"0\"><tbody>" +
						"<tr style=\"background: #EEEEEE;\">" +
						"<td><b>" + Catalog.GetString ("Subjects") + "</b></td>" +
						MarksStr + SemesterMarksStr + DescriptiveMarks +
						"</tr>" + rtn + "</tbody></table>" + BehaviourMarkStr + "<br/>";
			}

			return "";
		}


		/// <summary>Gets attendance</summary>
		private string GetAttendance (int StudentID)
		{
			if (PrintAttend.Active) {
				string ShowStr = "";
				if (Report.Active == 0)
					ShowStr = "AND ((month >= '9' AND month <= '12') OR (month = '1'))";
				else
					ShowStr = "AND month >= '2' AND month <= '6'";
	
				
				IDataReader reader = DB.Query (String.Format ("SELECT * FROM " +
						"attendance WHERE student = '{0}' {1};", StudentID, ShowStr));
	
				int Ab = 0, Lat = 0, LOA = 0, NLOA = 0;
				bool ok = false;
				
				if (reader != null) {
					while (reader.Read ()) {
						Ab  += (int) reader ["absence"];
						Lat += (int) reader ["lateness"];
						LOA += (int) reader ["loabsence"];
			
						NLOA = Ab - LOA;

						ok = true;
					}
					
					reader.Close ();
				}
	
	
				if (ok == true)
					return String.Format ("<h2>" + Catalog.GetString ("Attendance") +
								"</h2><ul><li>" + Catalog.GetString ("Absences") +
								": {0}</li><li>" + Catalog.GetString ("Latenesses") +
								": {1}</li><li>" + Catalog.GetString ("Leave of absences") +
								": {2}</li><li>" + Catalog.GetString ("Unjustified hours") +
								": {3}</li></ul>", Ab.ToString (), Lat.ToString (),
								LOA.ToString (), NLOA.ToString ());
			}

			return "";
		}


		/// <summary>Gets remarks</summary>
		private string GetRemarks (int StudentID)
		{
			if (PrintRemarks.Active) {
				string pos = "", neg = "", rtn = "";
				IDataReader reader = DB.Query (String.Format ("SELECT * FROM " +
								"remarks WHERE student = '{0}';", StudentID));
				
				if (reader != null) {
					while (reader.Read ())
						if ((int) reader ["positive"] == 1)
							pos += "<li>" + (string) reader ["text"] + "</li>";
						else
							neg += "<li>" + (string) reader ["text"] + "</li>";
				
					reader.Close ();
				}
	
				if (pos != "")
					rtn += "<h2>" + Catalog.GetString ("Positive remarks") + "</h2><ul>\n" + pos + "</ul>";
				
				if (neg != "")
					rtn += "<h2>" + Catalog.GetString ("Negative remarks") + "</h2><ul>\n" + neg + "</ul>";
	
				if (rtn != "")
					return rtn;
			}

			return "";
		}
	}
}
