/*
 *  GUI/StudentDialog.cs
 *  Copyright (C) 2005-2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Data;
using System.Data.SqlTypes;
using System.Collections;
using Gtk;
using esofokles.Engine;
using esofokles.Widgets;
using Mono.Unix;


namespace esofokles.GUI {
	
	/// <summary>Student dialog</summary>
	public class StudentDialog {
		public Gtk.Dialog dialog;

		[Glade.Widget] private Gtk.Notebook       MainNotebook, MarksNotebook;
		[Glade.Widget] public  Gtk.Entry          Name, Surname, Mark, Description,
												  SID, Pesel, BirthPlace, Address,
												  ParentsAddress, Telephone,
												  Father, Mother, BirthDate;
		[Glade.Widget] public  Gtk.TextView       Info;
		[Glade.Widget] private Gtk.Label          PositiveRemarksCount, NegativeRemarksCount,
												  NameLabel, MarksLabel, NotLeaveOfAbsence,
												  YearAbsence1, YearLateness1, YearLOA1,
												  YearNLOA1, YearAbsence2, YearLateness2,
												  YearLOA2, YearNLOA2, AddMarkBtnLbl,
												  ClassName, AverageS1, AverageS2,
												  SemMarkLbl, BehMarkLbl;
		[Glade.Widget] private Gtk.Button         CancelBtn, RemarksRemoveBtn,
												  RemarksModifyBtn, AddMarkBtn,
												  RemoveMarkBtn, NewMarkBtn,
												  RemarksAddBtn;
		[Glade.Widget] private Gtk.TreeView       RemarksList, MarksList;
		[Glade.Widget] private Gtk.ComboBox       AttendanceMonth, Subjects, SemesterMark,
												  BehaviourMark;
		[Glade.Widget] private Gtk.SpinButton     AttendanceAbsence, Lateness, LeaveOfAbsence;
		[Glade.Widget] private Gtk.CheckButton    LatenessIsLOA;
		[Glade.Widget] private Gtk.Table          MarkTable;
		[Glade.Widget] private Gtk.RadioButton    Semester, Semester2;
		[Glade.Widget] private Gtk.TextView       DescriptiveMarkText;
		
		public  int        ClassID, StudentID;
		private int        AttendSelectedMonth, CurrentSemester;
		private int        CurrentSubject = -1, CurrentSemestermark;
		private bool       Init;
		private ArrayList  SubjectsList;
		private DateEdit   Date;
		private Glade.XML  gui;
		private Button24   OkBtn;
		private MessageBox InvalidPeselMsg, BoldFieldsMsg;


		enum SemesterMarkEnum {
			None            = 0,
			DescriptiveMark = 1,
			Empty1          = 2,
			Insufficient    = 3,
			Poor            = 4,
			Sufficient      = 5,
			Good            = 6,
			VeryGood        = 7,
			Excellent       = 8,
			Empty2          = 9,
			Discharged      = 10,
			Ungraded        = 11
		}
		
		
		enum MainNotebookEnum {
			Summary      = 0,
			PersonalData = 1,
			MarksList    = 2,
			Attendance   = 3,
			Remarks      = 4
		}
		
		
		struct RemarksStats {
			public static int Positive, Negative;
		}
		
		
		/// <summary>Constructor. Creates student dialog.</summary>
		public StudentDialog (Gtk.Window parent, int StudentID, int ClassID)
		{
			Init = true;
			
			gui = Glade.XML.FromAssembly ("gui.glade", "StudentDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("StudentDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;
			dialog.DestroyWithParent = true;
			
			this.StudentID = StudentID;
			this.ClassID   = ClassID;
			
			if (Tools.GetCurrentSemester () == 1)
				Semester.Active = true;
			else
				Semester2.Active = true;
			
			
			// OkBtn
			OkBtn = new Button24 (Catalog.GetString ("OK"));
			OkBtn.Clicked += new EventHandler (on_OkBtn_clicked);
			dialog.ActionArea.Add (OkBtn);
			OkBtn.Sensitive = false;
			OkBtn.ShowAll ();
			
			
			// Icons and colors
			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.STUDENT_HEAD, "StudentImg", gui);
			Icons.AssignImage (Icons.ADD_REMARK_BUTTON, "RemarksAddBtnImg", gui);
			Icons.AssignImage (Icons.REMOVE_REMARK_BUTTON, "RemarksRemoveBtnImg", gui);
			Icons.AssignImage (Icons.MODIFY_REMARK_BUTTON, "RemarksModifyBtnImg", gui);
			Icons.AssignImage (Icons.ADD_MARK_BUTTON, "AddMarkBtnImg", gui);
			Icons.AssignImage (Icons.REMOVE_MARK_BUTTON, "RemoveMarkBtnImg", gui);
			Icons.AssignImage (Icons.NEW_MARK_BUTTON, "NewMarkBtnImg", gui);
			Icons.AssignImage (Icons.OK_BUTTON, OkBtn.Img);
			Icons.AssignImage (Icons.CANCEL_BUTTON, "CancelBtnImg", gui);
			Icons.AssignImage (Icons.SUBJECTS_LIST_BUTTON, "SubjectsListBtnImg", gui);
			Icons.AssignImage (Icons.SUMMARY_TAB, "SummaryImg", gui);
			Icons.AssignImage (Icons.PERSONAL_DATA, "PersonalDataImg", gui);
			Icons.AssignImage (Icons.MARKS, "MarksListImg", gui);
			Icons.AssignImage (Icons.ATTENDANCE, "AttendanceImg", gui);
			Icons.AssignImage (Icons.REMARKS, "RemarksImg", gui);
			Icons.AssignImage (Icons.NEXT_BUTTON, "MoreBtnImg2", gui);
			Icons.AssignImage (Icons.NEXT_BUTTON, "MoreBtnImg3", gui);
			Icons.AssignImage (Icons.NEXT_BUTTON, "MoreBtnImg4", gui);
			
			Colors.Modify (Colors.HEAD, "NameViewport", gui);
			Colors.Modify (Colors.BG, "SummaryBg", gui);
			
			
			// Messages in window
			BoldFieldsMsg = new MessageBox (MessageBox.MsgType.info, false, 8, 300,
				Catalog.GetString ("<b>Bold fields</b> must be filled in !"));
			BoldFieldsMsg.AddToBox ("PersonalDataPage", gui);
			BoldFieldsMsg.Hide ();
			
			new MessageBox (MessageBox.MsgType.info, false, 0, 300,
				Catalog.GetString ("If you want to add a subject to above list, "
					+ "click on <u>Subjects list</u> button.")).AddToBox ("info1", gui);
			
			InvalidPeselMsg = new MessageBox (MessageBox.MsgType.error, false, 8,
							200, Catalog.GetString ("Invalid PESEL number !"));
			InvalidPeselMsg.AddToBox ("PersonalDataPage", gui);
			InvalidPeselMsg.Hide ();
			
			
			// Class name
			IDataReader reader3 = DB.Query (String.Format ("SELECT * FROM classes where id = {0};", ClassID));
			if (reader3 != null) {
				if (reader3.Read ())
					ClassName.Text = String.Format (Catalog.GetString ("Class: {0} {1}, Educator: {2}"),
						reader3 ["number"], (string) reader3 ["token"],
						(string) reader3 ["description"]);
				
				reader3.Close ();
			}
			
			
			// Creating lists
			RemarksListCreate ();
			MarksListCreate ();
			ReloadBehaviourMark ();

			Date = new DateEdit ();
			MarkTable.Attach (Date, 2, 3, 1, 2);
			

			if (StudentID == -1)
				MainNotebook.Page = (int) MainNotebookEnum.PersonalData;
			
			Tools.SetSensitive (dialog, false);
			

			if (StudentID == -1) {
				CancelBtn.Show ();
				BoldFieldsMsg.Show ();
			} else {
				CancelBtn.Hide ();
				
				IDataReader reader = DB.Query (String.Format ("SELECT * FROM students WHERE id = '{0}';",
								StudentID));
				if (reader != null) {
					reader.Read ();
					Name.Text           = (string) reader ["name"];
					Surname.Text        = (string) reader ["surname"];
					Pesel.Text          = (string) reader ["pesel"];
					SID.Text            = (string) reader ["sid"];
					BirthPlace.Text     = (string) reader ["birthplace"];
					Address.Text        = (string) reader ["address"];
					ParentsAddress.Text = (string) reader ["parentsaddress"];
					Telephone.Text      = (string) reader ["telephone"];
					Father.Text         = (string) reader ["father"];
					Mother.Text         = (string) reader ["mother"];
					Info.Buffer.Text    = (string) reader ["info"];
					BirthDate.Text      = (string) reader ["birthdate"];
				}
				
				reader.Close ();
				ReloadRemarksList ();
				
				LatenessIsLOA.Active = esofokles.Engine.Settings.GetBool ("LatenessIsLOA");
				
				SetSensitive ();
			}

			
			// Autocompletion
			Autocompletion.Assign (Name, Field.StudentsName);
			Autocompletion.Assign (BirthPlace, Field.StudenstBirthplace);
			Autocompletion.Assign (Description, Field.MarksComment);
			
			
			// Subjects list 
			SubjectsList = new ArrayList ();
			Subjects.AppendText (Catalog.GetString ("Behaviour"));
			SubjectsList.Add    (0);
				
			IDataReader reader2 = DB.Query (String.Format ("SELECT * FROM subjects WHERE class = '{0}' ORDER by subject;", ClassID));
			if (reader2 != null) {
				while (reader2.Read ()) {
					Subjects.AppendText ((string) reader2 ["subject"]);
					SubjectsList.Add ((int) reader2 ["id"]);
				}

				reader2.Close ();
				
				Gtk.TreeIter tmp;
				if (Subjects.Model.GetIterFirst (out tmp))
					Subjects.Active = 0;
				
				LoadSchoolYearAttendanceSummary ();
				LoadRemarksSummary ();
				LoadAverage ();
			}
			
			SetSelectedMonth (DateTime.Today.Month);
				
			Init = false;
		}
		
		
		protected void on_OkBtn_clicked (object o, EventArgs args)
		{
			if (StudentID != -1) {
				IDataReader reader2 = DB.Query (String.Format ("SELECT * FROM students WHERE pesel = '{0}' AND id != '{1}' AND class = ANY(select id from classes where year = ANY(SELECT year from classes where id = '{2}'));", Pesel.Text, StudentID, ClassID));
				if (reader2 != null) {
					if (reader2.Read ()) {	
						Tools.SetSensitive (dialog, false);
						MainNotebook.Page = (int) MainNotebookEnum.PersonalData;
						
						MessageDialog md = new MessageDialog (this.dialog, DialogFlags.DestroyWithParent, MessageType.Error,
						ButtonsType.Close, Catalog.GetString ("Student with the same PESEL number already exists !"));
						md.Run ();
						md.Destroy ();
						
						Tools.SetSensitive (dialog, true);
						dialog.Focus = Pesel;
						
						reader2.Close ();
						return;
					}

					reader2.Close ();
				}
				
				SaveAttendance (AttendSelectedMonth);
				SemesterMarkSave ();
				
				if (CurrentSubject == -1)
					SaveBehaviourMark ();
			} else {
				IDataReader reader2 = DB.Query (String.Format ("SELECT * FROM students WHERE pesel = '{0}' AND class = ANY(select id from classes where year = ANY(SELECT year from classes where id = '{1}'));", Pesel.Text, ClassID));
				if (reader2 != null) {
					if (reader2.Read ()) {	
						Tools.SetSensitive (dialog, false);
						MainNotebook.Page = (int) MainNotebookEnum.PersonalData;
						
						MessageDialog md = new MessageDialog (this.dialog, DialogFlags.DestroyWithParent, MessageType.Error,
						ButtonsType.Close, Catalog.GetString ("Student with the same PESEL number already exists !"));
						md.Run ();
						md.Destroy ();
						
						Tools.SetSensitive (dialog, true);
						dialog.Focus = Pesel;
						
						reader2.Close ();
						return;
					}

					reader2.Close ();
				}
			}
			
			
			Autocompletion.AddText (Name, Field.StudentsName);
			Autocompletion.AddText (BirthPlace, Field.StudenstBirthplace);
			
			dialog.Respond ((int) ResponseType.Ok);
		}
		
 
		protected void on_MainNotebook_switch_page (object o, SwitchPageArgs args)
		{
			if (MainNotebook.Page == (int) MainNotebookEnum.PersonalData)
				return;
			
			if ((Name.Text == "") || (Surname.Text == "") || (Pesel.Text == "")) {
				Tools.SetSensitive (dialog, false);
				MainNotebook.Page = (int) MainNotebookEnum.PersonalData;

				MessageDialog md = new MessageDialog (this.dialog, DialogFlags.DestroyWithParent, MessageType.Warning,
					ButtonsType.Close, Catalog.GetString (
					"First you must fill <b>First name</b>, <b>Last name</b> and <b>PESEL</b> fields !"));
				md.Run ();
				md.Destroy ();
				Tools.SetSensitive (dialog, true);
				
				return;
			}
			
			IDataReader reader2 = DB.Query (String.Format ("SELECT * FROM students WHERE pesel = '{0}' AND id != '{1}' AND class = ANY(select id from classes where year = ANY(SELECT year from classes where id = '{2}'));", Pesel.Text, StudentID, ClassID));
			if (reader2 != null) {
				if (reader2.Read ()) {	
					Tools.SetSensitive (dialog, false);
					MainNotebook.Page = (int) MainNotebookEnum.PersonalData;
					
					MessageDialog md = new MessageDialog (this.dialog, DialogFlags.DestroyWithParent, MessageType.Error,
					ButtonsType.Close, Catalog.GetString ("Student with the same PESEL number already exists !"));
					md.Run ();
					md.Destroy ();
					
					Tools.SetSensitive (dialog, true);
					dialog.Focus = Pesel;
					
					reader2.Close ();
					return;
				}

				reader2.Close ();
			}

			
			if (StudentID == -1) {
				Tools.SetSensitive (dialog, false);
				
				DB.FastQuery (String.Format ("INSERT INTO students (class, name, surname, pesel, sid, birthdate, birthplace, address, parentsaddress, telephone, father, mother, info) VALUES ({0}, '{1}', '{2}', '{3}', '{4}', '{5}', '{6}', '{7}', '{8}', '{9}', '{10}', '{11}', '{12}');",
					ClassID, DB.Escape (Name.Text), DB.Escape (Surname.Text),
					DB.Escape (Pesel.Text), DB.Escape (SID.Text), DB.Escape (BirthDate.Text),
					DB.Escape (BirthPlace.Text), DB.Escape (Address.Text),
					DB.Escape (ParentsAddress.Text), DB.Escape (Telephone.Text),
					DB.Escape (Father.Text), DB.Escape (Mother.Text), DB.Escape (Info.Buffer.Text)));
				
				
				IDataReader read = DB.Query (String.Format (
					"SELECT id from students where (class = '{0}') AND (surname = '{1}') AND (pesel = '{2}');",
						ClassID, DB.Escape (Surname.Text), DB.Escape (Pesel.Text)));
				
				if (read != null) {
					read.Read ();
					StudentID  = (int) read ["id"];
					
					read.Close ();
				}
				
				CancelBtn.Hide ();
				Tools.SetSensitive (dialog, true);
			}
			
			
			if (MainNotebook.Page == (int) MainNotebookEnum.Summary) {
				Tools.SetSensitive (dialog, false);
				SemesterMarkSave ();
				SaveAttendance (AttendSelectedMonth);
				LoadSchoolYearAttendanceSummary ();
				LoadRemarksSummary ();
				if (CurrentSubject == -1)
					SaveBehaviourMark ();
				LoadAverage ();
				Tools.SetSensitive (dialog, true);
			}
		}
	

		
		// Personal data tab ******************************************
		
		/// <summary>Sets sensitive</summary>
		private void SetSensitive ()
		{
			OkBtn.Sensitive = (Name.Text != "") && (Surname.Text != "") && (Pesel.Text != "");
			BoldFieldsMsg.Visible = (Name.Text == "") || (Surname.Text == "") || (Pesel.Text == "");
		}


		protected void on_Name_changed (object o, EventArgs args)
		{
			SetSensitive ();
			NameLabel.Markup = "<span weight='bold' size='x-large'>" + Name.Text + " " + Surname.Text + "</span>";
		}


		protected void on_Surname_changed (object o, EventArgs args)
		{
			SetSensitive ();
			NameLabel.Markup = "<span weight='bold' size='x-large'>" + Name.Text + " " + Surname.Text + "</span>";
		}


		protected void on_Pesel_changed (object o, EventArgs args)
		{
			SetSensitive ();
			
			if (Pesel.Text.Length == 11)
				if (Tools.CheckPesel (Pesel.Text)) {
					Pesel.ModifyText (Gtk.StateType.Normal);
					Pesel.ModifyBase (Gtk.StateType.Normal);
					InvalidPeselMsg.Hide ();
				} else {
					Pesel.ModifyBase (Gtk.StateType.Normal, new Gdk.Color (255, 185, 185));
					InvalidPeselMsg.Show ();
				}
			else {
				Pesel.ModifyText (Gtk.StateType.Normal);
				Pesel.ModifyBase (Gtk.StateType.Normal, new Gdk.Color (255, 255, 226));
				InvalidPeselMsg.Hide ();
			}
		}


		protected void on_ParentsAddress_focus_in_event (object o, FocusInEventArgs args)
		{
			if (ParentsAddress.Text == "")
				ParentsAddress.Text = Address.Text;
		}
		
		
		protected void on_Pesel_focus_out_event (object o, FocusOutEventArgs args)
		{
			if ((BirthDate.Text == "") && (Tools.CheckPesel (Pesel.Text)))
				BirthDate.Text = Tools.GetDateFromPesel (Pesel.Text);
		}
		
		
		protected void on_MoreAttendance_clicked (object o, EventArgs args)
		{
			MainNotebook.Page = (int) MainNotebookEnum.Attendance;
		}
		
		
		protected void on_MoreAverage_clicked (object o, EventArgs args)
		{
			MainNotebook.Page = (int) MainNotebookEnum.MarksList;
		}
		
		
		protected void on_MoreRemarks_clicked (object o, EventArgs args)
		{
			MainNotebook.Page = (int) MainNotebookEnum.Remarks;
		}
		
		
		/// <summary>Loads remarks summary.</summary>
		private void LoadRemarksSummary ()
		{
			NegativeRemarksCount.Text = RemarksStats.Negative.ToString ();
			PositiveRemarksCount.Text = RemarksStats.Positive.ToString ();
		}
		
		
		/// <summary>Loads average.</summary>
		private void LoadAverage ()
		{
			IDataReader reader = DB.Query (String.Format ("SELECT semestermark" +
				"s.mark,semestermarks.semester,subjects.average FROM semestermarks,subjects WHERE " +
				"semestermarks.student = '{0}' AND subjects.average = '1' AND " +
				"semestermarks.subject = subjects.id AND (semestermarks.subject != '0');", StudentID));
			
			float semester1 = 0, semester2 = 0;
			int   s1cnt = 0, s2cnt = 0, tmp = 0;
			
			if (reader != null) {
				while (reader.Read ()) {
					if (((int) reader ["mark"] >= 2) && ((int) reader ["mark"] <= 6))
						tmp = (int) reader ["mark"];
					else
						continue;
					
					if ((int) reader ["semester"] == 0) {
						semester1 += tmp;
						s1cnt++;
					} else {
						semester2 += tmp;
						s2cnt++;
					}
				}

				reader.Close ();
			}
			
			semester1 /= s1cnt;
			semester2 /= s2cnt;
			
			if (s1cnt > 0)
				AverageS1.Text = String.Format ("{0}", System.Decimal.Round (Convert.ToDecimal (semester1), 2));
			else
				AverageS1.Text = Catalog.GetString ("None");
			
			if (s2cnt > 0)
				AverageS2.Text = String.Format ("{0}", System.Decimal.Round (Convert.ToDecimal (semester2), 2));
			else
				AverageS2.Text = Catalog.GetString ("None");
		}
		
		
		
		// Marks list tab *********************************************
		
		/// <summary>Creates marks list.</summary>
		private void MarksListCreate ()
		{
			Gtk.ListStore store = new Gtk.ListStore (typeof (string), typeof (string), typeof (string), typeof (int));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			MarksList.Model = store;

			MarksList.AppendColumn (Catalog.GetString ("Date"),    new CellRendererText (), "text", 0);
			MarksList.AppendColumn (Catalog.GetString ("Mark"),    new CellRendererText (), "text", 1);
			MarksList.AppendColumn (Catalog.GetString ("Comment"), new CellRendererText (), "text", 2);

			MarksList.Columns [0].Clickable = true;
			MarksList.Columns [0].SortIndicator = true;
			MarksList.Columns [0].SortColumnId = 0;

			MarksList.Columns [1].Clickable = true;
			MarksList.Columns [1].SortIndicator = true;
			MarksList.Columns [1].SortColumnId = 1;

			MarksList.Columns [2].Clickable = true;
			MarksList.Columns [2].SortIndicator = true;
			MarksList.Columns [2].SortColumnId = 2;
		}


		/// <summary>Reloads marks list.</summary>
		private void ReloadMarksList ()
		{
			if (Subjects.Active == -1)
				return;
			
			IDataReader reader = DB.Query (String.Format ("SELECT * FROM marks " +
						"WHERE student = '{0}' AND subject = '{1}' AND semester = " +
						"'{2}';", StudentID, DB.Escape (SubjectsList [Subjects.Active].ToString ()), CurrentSemester));
			
			if (reader != null) {
				Gtk.ListStore store = (Gtk.ListStore) MarksList.Model;
				store.Clear ();
				
				while (reader.Read ())
					store.AppendValues (new object [] {(string) reader ["date"], (string) reader ["mark"],
							(string) reader ["description"], (int) reader ["id"]});

				reader.Close ();
			}
		}
		
		
		/// <summary>Reloads Behaviour mark.</summary>
		private void ReloadBehaviourMark ()
		{
			IDataReader reader2 = DB.Query (String.Format ("SELECT * FROM " +
				"semestermarks WHERE student = '{0}' AND subject = '0' AND " +
					"semester = '{1}';", StudentID, CurrentSemester));
			
			if (reader2 != null) {
				if (reader2.Read ()) {
					BehaviourMark.Active = (int) reader2 ["mark"];
					
				} else
					BehaviourMark.Active = 0;
				
				reader2.Close ();
			}
		}


		/// <summary>Reloads semester marks.</summary>
		private void ReloadSemesterMark ()
		{
			Gtk.TreeIter iter;
			if (Subjects.GetActiveIter (out iter)) {
				IDataReader reader= DB.Query (String.Format ("SELECT * FROM " +
					"semestermarks WHERE student = '{0}' AND subject = '{1}' AND " +
						"semester = '{2}';", StudentID, DB.Escape (SubjectsList [Subjects.Active].ToString ()),
						CurrentSemester));
				
				if (reader != null) {
					if (reader.Read ()) {
						if ((int) reader ["mark"] == -1)
							SemesterMark.Active = (int) SemesterMarkEnum.None;
						else if ((int) reader ["mark"] == -2)
							SemesterMark.Active = (int) SemesterMarkEnum.Discharged;
						else if ((int) reader ["mark"] == -3)
							SemesterMark.Active = (int) SemesterMarkEnum.Ungraded;
						else if ((int) reader ["mark"] == -4)
							SemesterMark.Active = (int) SemesterMarkEnum.DescriptiveMark;
						else if (((int) reader ["mark"] >= (int) SemesterMarkEnum.Insufficient - 2) &&
								 ((int) reader ["mark"] <= (int) SemesterMarkEnum.Excellent - 2))
							SemesterMark.Active = (int) reader ["mark"] + 2;
						else
							SemesterMark.Active = (int) SemesterMarkEnum.None;
						
						DescriptiveMarkText.Buffer.Text = (string) reader ["description"];
					} else
						SemesterMark.Active = (int) SemesterMarkEnum.None;
					
					reader.Close ();
				}
			}
		}


		/// <summary>Gets semester mark.</summary>
		private double GetSemesterMark ()
		{
			if (SemesterMark.Active == (int) SemesterMarkEnum.None)
				return -1;
			else if (SemesterMark.Active == (int) SemesterMarkEnum.Discharged)
				return -2;
			else if (SemesterMark.Active == (int) SemesterMarkEnum.Ungraded)
				return -3;
			else if (SemesterMark.Active == (int) SemesterMarkEnum.DescriptiveMark)
				return -4;
			else
				return SemesterMark.Active - 2;
		}
		
		
		/// <summary>Saves Behaviour mark.</summary>
		private void SaveBehaviourMark ()
		{
			IDataReader reader2 = DB.Query(String.Format ("SELECT * FROM " +
				"semestermarks WHERE student = '{0}' AND subject = '0' AND semester = '{1}';",
				StudentID, CurrentSemester));
			
			if (reader2 != null)
				if (reader2.Read ()) {
					reader2.Close ();
					DB.FastQuery (String.Format ("UPDATE semestermarks SET " +
						"mark = '{0}', description = '{1}' WHERE student = " +
						"'{2}' AND subject = '0' AND semester = '{3}';",
						BehaviourMark.Active, "", StudentID, CurrentSemester));
				} else {
					reader2.Close ();
					DB.FastQuery (String.Format ("INSERT INTO semestermarks" +
						"(semester, student, subject, mark, description) VALUES('{0}', '{1}', '0', '{2}', '');",
						CurrentSemester, StudentID, BehaviourMark.Active));
				}
		}


		/// <summary>Saves semester mark.</summary>
		private void SemesterMarkSave ()
		{
			if (CurrentSubject != -1) {
				IDataReader reader = DB.Query(String.Format ("SELECT * FROM " +
					"semestermarks WHERE student = '{0}' AND subject = '{1}' AND semester = '{2}';",
					StudentID, SubjectsList [CurrentSubject], CurrentSemester));
				
				if (reader != null)
					if (reader.Read ()) {
						reader.Close ();
						DB.FastQuery (String.Format ("UPDATE semestermarks SET " +
							"mark = '{0}', description = '{1}' WHERE student = " +
							"'{2}' AND subject = '{3}' AND semester = '{4}';",
							GetSemesterMark (), DB.Escape (DescriptiveMarkText.Buffer.Text),
							StudentID, SubjectsList [CurrentSubject],
							CurrentSemester));
					} else {
						reader.Close ();
						DB.FastQuery (String.Format ("INSERT INTO semestermarks" +
							"(semester, student, subject, mark, description) VALUES('{0}', '{1}', '{2}', '{3}', '{4}');",
							CurrentSemester, StudentID, SubjectsList [CurrentSubject],
							GetSemesterMark (), DB.Escape (DescriptiveMarkText.Buffer.Text)));
					}
				

				SaveBehaviourMark ();
			}
		}
		
		
		protected void on_SemesterMark_changed (object o, EventArgs args)
		{
			MarksLabel.Visible = !(SemesterMark.Active == (int) SemesterMarkEnum.DescriptiveMark);
			
			if (SemesterMark.Active == (int) SemesterMarkEnum.DescriptiveMark)
				MarksNotebook.Page = 1;
			else
				MarksNotebook.Page = 0;
			
			if ((SemesterMark.Active == (int) SemesterMarkEnum.Empty1) ||
				(SemesterMark.Active == (int) SemesterMarkEnum.Empty2))
				
				if (CurrentSemestermark < SemesterMark.Active)
					SemesterMark.Active++;
				else
					SemesterMark.Active--;
			
			CurrentSemestermark = SemesterMark.Active;
		}


		protected void on_Subjects_changed (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			
			SemesterMarkSave ();
			ClearMarksFields ();
		
			Gtk.TreeIter iter;
			if (Subjects.GetActiveIter (out iter)) {
				CurrentSubject = Subjects.Active;
			
				ReloadMarksList ();
				ReloadSemesterMark ();
	
				MarkTable.Sensitive = true;
				MarksList.Sensitive = true;
				
				BehMarkLbl.Visible    = Subjects.Active == 0;
				SemMarkLbl.Visible    = Subjects.Active != 0;
				SemesterMark.Visible  = Subjects.Active != 0;
				BehaviourMark.Visible = Subjects.Active == 0;
			} else {
				MarkTable.Sensitive = false;
				MarksList.Sensitive = false;
			}
			
			Tools.SetSensitive (dialog, true);
		}


		protected void on_SubjectsListBtn_clicked (object o, EventArgs args)
		{
			SemesterMarkSave ();
			
			SubjectsListDialog dlg = new SubjectsListDialog (this.dialog, ClassID);
			Tools.SetSensitive (dialog, false);
			
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				Gtk.ListStore store = (Gtk.ListStore) dlg.SubjectsList.Model;
				Gtk.TreeIter  iter;
	
				while (Subjects.Model.GetIterFirst (out iter))
					Subjects.RemoveText (0);
	
				SubjectsList = new ArrayList ();
				Subjects.AppendText (Catalog.GetString ("Behaviour"));
				SubjectsList.Add    (0);
				
				for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
					Subjects.AppendText ((string) store.GetValue (iter, 0));
					SubjectsList.Add ((int) store.GetValue (iter, 2));
				}

				if (Subjects.Model.GetIterFirst (out iter)) {
					Subjects.Active = 0;
					MarkTable.Sensitive = true;
					MarksList.Sensitive = true;
				} else {
					MarkTable.Sensitive = false;
					MarksList.Sensitive = false;
				}
			}
			
			dlg.dialog.Destroy ();
			Tools.SetSensitive (dialog, true);
		}


		protected void on_ShowFirstSemester_toggled (object o, EventArgs args)
		{
			if (Init == true) {
				CurrentSemester = Convert.ToInt32 (! Semester.Active);
				return;
			}
			
			Tools.SetSensitive (dialog, false);
			
			SemesterMarkSave ();
			SaveBehaviourMark ();
			CurrentSemester = Convert.ToInt32 (! Semester.Active);
			
			Gtk.TreeIter iter;
			if (Subjects.GetActiveIter (out iter))
				CurrentSubject = Subjects.Active;
			
			ReloadMarksList ();
			ReloadSemesterMark ();
			ReloadBehaviourMark ();
			
			Tools.SetSensitive (dialog, true);
		}


		protected void on_Mark_changed (object o, EventArgs args)
		{
			if (Mark.Text != "")
				MarksBtnsSetSensitive (1, -1, 1);
			else
				MarksBtnsSetSensitive (0, -1, -1);
		}


		protected void on_Mark_activate (object o, EventArgs args)
		{
			on_AddMarkBtn_clicked (null, null);
		}


		protected void on_Description_activate (object o, EventArgs args)
		{
			on_AddMarkBtn_clicked (null, null);
		}


		protected void on_MarksList_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;

			if (MarksList.Selection.GetSelected (out mod, out itr)) {
				ClearMarksFields ();
				
				Date.SetDate ((string) mod.GetValue (itr, 0));
				Mark.Text        = (string) mod.GetValue (itr, 1);
				Description.Text = (string) mod.GetValue (itr, 2);
				
				MarksBtnsSetSensitive (-1, 1, 1);
				SetButtonSave (true);
			} else {
				MarksBtnsSetSensitive (-1, 0, 1);
				SetButtonSave (false);
			}
		}


		protected void on_NewMarkBtn_clicked (object o, EventArgs args)
		{
			ClearMarksFields ();
			MarksBtnsSetSensitive (0, 0, 0);
			SetButtonSave (false);
			dialog.Focus = Mark;
		}


		protected void on_AddMarkBtn_clicked (object o, EventArgs args)
		{
			if (Mark.Text == "")
				return;
			
			Gtk.TreeIter iter;
			Subjects.GetActiveIter (out iter);
			
			Tools.SetSensitive (dialog, false);
			
			if (AddMarkBtnLbl.Text == Catalog.GetString ("Save")) {
				Gtk.TreeModel mod;
				Gtk.TreeIter  itr;
				
				if (MarksList.Selection.GetSelected (out mod, out itr)) {
					string query = "";
					query += String.Format ("UPDATE marks SET mark = '{0}' WHERE id = '{1}';",
								DB.Escape (Mark.Text), (int) mod.GetValue (itr, 3));
					query += String.Format ("UPDATE marks SET description = '{0}' WHERE id = '{1}';",
								DB.Escape (Description.Text), (int) mod.GetValue (itr, 3));
					query += String.Format ("UPDATE marks SET date = '{0}' WHERE id = '{1}';",
								Date.GetDateString (), (int) mod.GetValue (itr, 3));
					DB.FastQuery (query);
				}
			} else
				DB.FastQuery (String.Format ("INSERT INTO marks (student, semester, subject, mark, date, description) VALUES('{0}', " +
						"'{1}', '{2}', '{3}', '{4}', '{5}');",
						StudentID, CurrentSemester, DB.Escape (SubjectsList [Subjects.Active].ToString ()),
						DB.Escape (Mark.Text), Date.GetDateString (), DB.Escape (Description.Text)));

			Autocompletion.AddText (Description, Field.MarksComment);
			ReloadMarksList ();
			ClearMarksFields ();
			SetButtonSave (false);
			
			Tools.SetSensitive (dialog, true);
			
			this.dialog.Focus = Mark;
		}


		protected void on_RemoveMarkBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (MarksList.Selection.GetSelected (out mod, out itr)) {
				Tools.SetSensitive (dialog, false);
				
				DB.FastQuery (String.Format ("DELETE FROM marks WHERE id = '{0}';", mod.GetValue (itr, 3).ToString ()));
	
				ReloadMarksList ();
				ClearMarksFields ();
				SetButtonSave (false);
				MarksBtnsSetSensitive (0, 0, 0);
				
				Tools.SetSensitive (dialog, true);
			}
		}


		/// <summary>Sets marks buttons sensitive.</summary>
		private void MarksBtnsSetSensitive (int Add, int Remove, int New)
		{
			if (Add == 1)
				AddMarkBtn.Sensitive = true;
			else if (Add == 0)
				AddMarkBtn.Sensitive = false;

			if (Remove == 1)
				RemoveMarkBtn.Sensitive = true;
			else if (Remove == 0)
				RemoveMarkBtn.Sensitive = false;

			if (New == 1)
				NewMarkBtn.Sensitive = true;
			else if (New == 0)
				NewMarkBtn.Sensitive = false;
		}


		/// <summary>Clears marks fields.</summary>
		private void ClearMarksFields ()
		{
			Date.SetDate (DateTime.Today);
			Mark.Text = "";
			Description.Text = "";
		}


		/// <summary>Sets buttons save (text and icon.)</summary>
		private void SetButtonSave (bool Save)
		{
			if (Save == true) {
				AddMarkBtnLbl.Text = Catalog.GetString ("Save");
				Icons.AssignImage (Icons.SAVE_MARK_BUTTON, "AddMarkBtnImg", gui);
			} else {
				AddMarkBtnLbl.Text = Catalog.GetString ("Add mark");
				Icons.AssignImage (Icons.ADD_MARK_BUTTON, "AddMarkBtnImg", gui);
			}
		}
	


		// Attendance tab **********************************************
		protected void on_AttendanceAbsence_value_changed (object o, EventArgs args)
		{
			if (LatenessIsLOA.Active == true) {
				LeaveOfAbsence.SetRange (0, AttendanceAbsence.Value + Lateness.Value);
				NotLeaveOfAbsence.Text = String.Format ("{0}", (AttendanceAbsence.Value + Lateness.Value)
										- LeaveOfAbsence.Value);
			} else {
				LeaveOfAbsence.SetRange (0, AttendanceAbsence.Value);
				NotLeaveOfAbsence.Text = String.Format ("{0}", AttendanceAbsence.Value - LeaveOfAbsence.Value);
			}
		}


		protected void on_Lateness_value_changed (object o, EventArgs args)
		{
			if (LatenessIsLOA.Active == true) {
				LeaveOfAbsence.SetRange (0, AttendanceAbsence.Value + Lateness.Value);
				NotLeaveOfAbsence.Text = String.Format ("{0}", (AttendanceAbsence.Value + Lateness.Value)
										- LeaveOfAbsence.Value);
			}
		}


		protected void on_LeaveOfAbsence_value_changed (object o, EventArgs args)
		{
			if (LatenessIsLOA.Active == true)
				NotLeaveOfAbsence.Text = String.Format ("{0}", (AttendanceAbsence.Value + Lateness.Value)
										- LeaveOfAbsence.Value);
			else
				NotLeaveOfAbsence.Text = String.Format ("{0}", AttendanceAbsence.Value - LeaveOfAbsence.Value);
		}


		protected void on_LatenessIsLOA_toggled (object o, EventArgs args)
		{
			if (LatenessIsLOA.Active == true) {
				LeaveOfAbsence.SetRange (0, AttendanceAbsence.Value + Lateness.Value);
				NotLeaveOfAbsence.Text = String.Format ("{0}", (AttendanceAbsence.Value + Lateness.Value)
										- LeaveOfAbsence.Value);
			} else {
				LeaveOfAbsence.SetRange (0, AttendanceAbsence.Value);
				NotLeaveOfAbsence.Text = String.Format ("{0}", AttendanceAbsence.Value - LeaveOfAbsence.Value);
			}
			
			esofokles.Engine.Settings.SetBool ("LatenessIsLOA", LatenessIsLOA.Active);
		}
		
		
		/// <summary>Gets number of selected month.</summary>
		private int GetSelectedMonth ()
		{
			if ((AttendanceMonth.Active >= 1) && (AttendanceMonth.Active <= 4))
				return AttendanceMonth.Active + 8;
			else if (AttendanceMonth.Active > 4)
				return AttendanceMonth.Active - 4;
			else
				return 0;
		}
		
		
		/// <summary>Sets number of selected month.</summary>
		private void SetSelectedMonth (int MonthNumber)
		{
			if ((MonthNumber >= 9) && (MonthNumber <= 12))
				AttendanceMonth.Active = MonthNumber - 8;
			else if ((MonthNumber >= 1) && (MonthNumber <= 6))
				AttendanceMonth.Active = MonthNumber + 4;
			else
				AttendanceMonth.Active = 0;
		}


		/// <summary>Loads attendance.</summary>
		private void LoadAttendance (int month)
		{
			AttendanceAbsence.Sensitive = month != 0;
			Lateness.Sensitive         = month != 0;
			LeaveOfAbsence.Sensitive   = month != 0;
			
			if (month == 0) {
				AttendanceAbsence.Value = 0;
				Lateness.Value         = 0;
				LeaveOfAbsence.Value   = 0;
				NotLeaveOfAbsence.Text = "0";
				AttendSelectedMonth = 0;

				return;
			}


			IDataReader reader = DB.Query (String.Format ("SELECT * FROM attendance WHERE student = '{0}' AND month = '{1}';",
								StudentID, month));
			if (reader != null) {
				if (reader.Read ()) {
					AttendanceAbsence.Value = (int) reader ["absence"];
					Lateness.Value         = (int) reader ["lateness"];
					LeaveOfAbsence.Value   = (int) reader ["loabsence"];

					if (LatenessIsLOA.Active == true)
						NotLeaveOfAbsence.Text = String.Format ("{0}", ((int) reader ["absence"] + (int) reader ["lateness"])
										- (int) reader ["loabsence"]);
					else
						NotLeaveOfAbsence.Text = String.Format ("{0}", (int) reader ["absence"] - (int) reader ["loabsence"]);
				} else {
					AttendanceAbsence.Value = 0;
					Lateness.Value         = 0;
					LeaveOfAbsence.Value   = 0;
					NotLeaveOfAbsence.Text = "0";
				}
				
				reader.Close ();
			}

			AttendSelectedMonth = month;
		}


		/// <summary>Saves attendance.</summary>
		private void SaveAttendance (int month)
		{
			if (month == 0)
				return;

			
			IDataReader reader = DB.Query (String.Format ("SELECT * FROM attendance WHERE student = '{0}' AND month = '{1}';",
								StudentID, month));
			
			if (reader != null) {
				if (reader.Read ()) {
					reader.Close ();
					
					string query = "";
					query += String.Format ("UPDATE attendance SET absence = '{0}' WHERE student = '{1}' AND month = '{2}';",
							AttendanceAbsence.Value.ToString (), StudentID, month);
					query += String.Format ("UPDATE attendance SET lateness = '{0}' WHERE student = '{1}' AND month = '{2}';",
							Lateness.Value.ToString (), StudentID, month);
					query += String.Format ("UPDATE attendance SET loabsence = '{0}' WHERE student = '{1}' AND month = '{2}';",
							LeaveOfAbsence.Value.ToString (), StudentID, month);
					DB.FastQuery (query);
				} else {
					reader.Close ();
					DB.FastQuery (String.Format ("INSERT INTO attendance (student, month, absence, lateness, loabsence) VALUES('{0}', '{1}', '{2}', '{3}', '{4}');",
							StudentID, month, AttendanceAbsence.Value.ToString (),
							Lateness.Value.ToString (), LeaveOfAbsence.Value.ToString ()));
				}
			}
		}


		/// <summary>Loads school year attendance summary.</summary>
		private void LoadSchoolYearAttendanceSummary ()
		{
			IDataReader reader = DB.Query (String.Format ("SELECT * FROM attendance WHERE student = '{0}';",
								StudentID));

			int Ab1 = 0, Lat1 = 0, LOA1 = 0, NLOA1 = 0, Ab2 = 0, Lat2 = 0, LOA2 = 0, NLOA2 = 0, month = 0;
			
			if (reader != null) {
				while (reader.Read ()) {
					month = (int) reader ["month"];
					
					if ((((int) month >= 9) && (month <= 12)) || (month == 1)) {
						Ab1  += (int) reader ["absence"];
						Lat1 += (int) reader ["lateness"];
						LOA1 += (int) reader ["loabsence"];
						
						if (LatenessIsLOA.Active == true)
							NLOA1 = Ab1 + Lat1 - LOA1;
						else
							NLOA1 = Ab1 - LOA1;
					} else {
						Ab2  += (int) reader ["absence"];
						Lat2 += (int) reader ["lateness"];
						LOA2 += (int) reader ["loabsence"];
						
						if (LatenessIsLOA.Active == true)
							NLOA2 = Ab2 + Lat2 - LOA2;
						else
							NLOA2 = Ab2 - LOA2;
					}
				}
				
				reader.Close ();
			}

			
			YearAbsence1.Text  = Ab1.ToString ();
			YearLateness1.Text = Lat1.ToString ();
			YearLOA1.Text      = LOA1.ToString ();
			YearNLOA1.Text     = NLOA1.ToString ();
			
			YearAbsence2.Text  = Ab2.ToString ();
			YearLateness2.Text = Lat2.ToString ();
			YearLOA2.Text      = LOA2.ToString ();
			YearNLOA2.Text     = NLOA2.ToString ();
		}


		protected void on_AttendanceMonth_changed (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);
			SaveAttendance (AttendSelectedMonth);
			LoadAttendance (GetSelectedMonth ());
			Tools.SetSensitive (dialog, true);
		}



		// Remarks tab ************************************************
		protected void on_RemarksAddBtn_clicked (object o, EventArgs args)
		{
			Tools.SetSensitive (dialog, false);

			RemarkDialog dlg = new RemarkDialog (this.dialog, true,
						(Gtk.ListStore) RemarksList.Model, new Gtk.TreeIter ());
			dialog.Sensitive = false;

			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				DB.FastQuery (String.Format ("INSERT INTO remarks (student, positive, text) VALUES ('{0}', '{1}', '{2}');",
							StudentID, System.Convert.ToInt32 (dlg.RemarkGood.Active), DB.Escape (dlg.Remark.Buffer.Text)));
				ReloadRemarksList ();
			}
			
			dlg.dialog.Destroy ();
			dialog.Sensitive = true;
		
			Tools.SetSensitive (dialog, true);
			dialog.Focus = RemarksAddBtn;
		}


		protected void on_RemarksRemoveBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (RemarksList.Selection.GetSelected (out mod, out itr)) {
				Tools.SetSensitive (dialog, false);
				
				Gtk.ListStore store = (Gtk.ListStore) RemarksList.Model;

				MessageDialog msg = new MessageDialog (this.dialog, DialogFlags.DestroyWithParent, MessageType.Question,
						ButtonsType.YesNo, Catalog.GetString ("Are you really sure to remove selected remark ?"));
				
					if (ResponseType.Yes == (ResponseType) msg.Run ()) {
					RemarksButtonsSetSensitive (false);
										
					if (DB.FastQuery (String.Format ("DELETE FROM remarks WHERE id = {0};", (int) store.GetValue (itr, 2))))
						store.Remove (ref itr);
				}

				msg.Destroy ();
				Tools.SetSensitive (dialog, true);
			}
			
			dialog.Focus = RemarksList;
		}


		protected void on_RemarksList_row_activated (object o, RowActivatedArgs args)
		{
			on_RemarksModifyBtn_clicked (null, null);
		}


		protected void on_RemarksModifyBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (RemarksList.Selection.GetSelected (out mod, out itr)) {
				Tools.SetSensitive (dialog, false);
				
				RemarkDialog dlg = new RemarkDialog (this.dialog, false,
												(Gtk.ListStore) mod, itr);
				dialog.Sensitive = false;

				if (dlg.dialog.Run () == (int) ResponseType.Ok) {
					Gtk.ListStore store = (Gtk.ListStore) mod;
					
					string query = "";
					query += String.Format ("UPDATE remarks SET text='{0}' WHERE id = {1};",
							DB.Escape (dlg.Remark.Buffer.Text), (int) store.GetValue (itr, 2));
					query += String.Format ("UPDATE remarks SET positive='{0}' WHERE id = {1};",
							System.Convert.ToInt32 (dlg.RemarkGood.Active), (int) store.GetValue (itr, 2));
					DB.FastQuery (query);
					
					ReloadRemarksList ();
					RemarksButtonsSetSensitive (false);
				}
				
				dialog.Sensitive = true;
				dlg.dialog.Destroy ();
				Tools.SetSensitive (dialog, true);
			}
			
			dialog.Focus = RemarksList;
		}

	
		protected void on_RemarksList_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (RemarksList.Selection.GetSelected (out mod, out itr))
				RemarksButtonsSetSensitive (true);
			else
				RemarksButtonsSetSensitive (false);
		}


		/// <summary>Sets remarks buttons sensitive.</summary>
		private void RemarksButtonsSetSensitive (bool sensitive)
		{
			RemarksRemoveBtn.Sensitive = sensitive;
			RemarksModifyBtn.Sensitive = sensitive;
		}


		/// <summary>Creates remarks list.</summary>
		private void RemarksListCreate ()
		{
			Gtk.ListStore store = new Gtk.ListStore (typeof (string), typeof (string), typeof (int));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			RemarksList.Model = store;

			RemarksList.AppendColumn (Catalog.GetString ("Remark type"), new CellRendererText  (), "text", 0);
			RemarksList.AppendColumn (Catalog.GetString ("Remark"), new CellRendererText (), "text", 1);

			RemarksList.Columns [0].Clickable = true;
			RemarksList.Columns [0].SortIndicator = true;
			RemarksList.Columns [0].SortColumnId = 0;

			RemarksList.Columns [1].Clickable = true;
			RemarksList.Columns [1].SortIndicator = true;
			RemarksList.Columns [1].SortColumnId = 1;
		}


		/// <summary>Reloads remarks list.</summary>
		private void ReloadRemarksList ()
		{
			IDataReader reader = DB.Query (String.Format ("SELECT * FROM remarks WHERE student = '{0}';", StudentID));
			
			if (reader != null) {
				Gtk.ListStore store = (Gtk.ListStore) RemarksList.Model;
				store.Clear ();
				
				string remark;
				RemarksStats.Positive = 0;
				RemarksStats.Negative = 0;
				
				while (reader.Read ()) {
					if ((int) reader ["positive"] == 1) {
						remark = Catalog.GetString ("Positive");
						RemarksStats.Positive++;
					} else {
						remark = Catalog.GetString ("Negative");
						RemarksStats.Negative++;
					}
						
					store.AppendValues (new object [] {remark, (string) reader ["text"], (int) reader ["id"]});
				}

				reader.Close ();
			}
		}
	}
}
