/*
 *  Widgets/MessageBox.cs
 *  Copyright (C) 2006 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using GLib;
using System.Reflection;


namespace esofokles.Widgets {
	public class MessageBox : Gtk.HBox {
		
		public enum MsgType {
			info     = 0,
			error    = 1,
			question = 2,
			warning  = 3
		}
		
		
		public MessageBox (MsgType type, bool big, string message)
		{
			Create (type, big, 0, -1, message);
		}
		
		
		public MessageBox (MsgType type, bool big, string message, int width)
		{
			Create (type, big, 0, width, message);
		}
		
		
		public MessageBox (MsgType type, bool big, uint border, string message)
		{
			Create (type, big, border, -1, message);
		}
		
		
		public MessageBox (MsgType type, bool big, uint border, int width, string message)
		{
			Create (type, big, border, width, message);
		}
		
		
		public void Create (MsgType type, bool big, uint border, int width, string message)
		{
			if (border > 3)
				this.BorderWidth = border - 3;
			else
				this.BorderWidth = 0;
			
			Gtk.Viewport view = new Gtk.Viewport ();
			view.BorderWidth = 3;
			view.ModifyBg (Gtk.StateType.Normal, GUI.Colors.INFO);
			this.Add (view);
			
			Gtk.HBox MainBox = new Gtk.HBox ();
			view.Add (MainBox);
			
			string size = "";
			if (big)
				size = "48";
			else
				size = "20";
			
			Gtk.Image icon = new Gtk.Image ();
			switch (type) {
				case MsgType.error:
					icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (),
								String.Format ("error-{0}.png", size)));
				break;
				
				case MsgType.info:
					icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (),
								String.Format ("info-{0}.png", size)));
				break;
				
				case MsgType.question:
					icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (),
								String.Format ("question-{0}.png", size)));
				break;
				
				case MsgType.warning:
					icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (),
								String.Format ("warning-{0}.png", size)));
				break;
			}
			MainBox.PackStart (icon, false, false, 6);
			
			Gtk.Label msg = new Gtk.Label ();
			msg.UseMarkup = true;
			msg.Wrap      = true;
			msg.WidthRequest = width;
			
			if (big)
				msg.Markup = message;
			else
				msg.Markup = "<span size='small'>" + message + "</span>";
			
			msg.Xalign    = (float) 0.00;
			MainBox.PackEnd (msg, true, true, 6);
			
			this.ShowAll ();
		}
		
		
		public void AddToBox (string WidgetName, Glade.XML xml)
		{
			Gtk.Box widget = (Gtk.Box) xml.GetWidget (WidgetName);
			widget.PackEnd (this, false, false, 0);
			widget.ShowAll ();
		}
	}
}
